/* mp3t.cc 

   The MP3 tag command line editor

   (c) 2000 by Martin Pitt (Piware; www.piware.de)

   This program uses the class Tag of KMP3TE, written by Antoine
   Laydier (laydier@usa.net).

   This file is distributed under the terms and conditions of the "GNU
   General Public License" (see file COPYING). 

   Please send bug reports, questions and suggestions to:
   piware@piware.de
*/

#include "tag.h"
#include "mp3topts.h"
#include "mp3ttools.h"
#include <getopt.h> // optind
#include <string.h> // strncmp()
#include <libintl.h> // gettext()
#include <locale.h> // LC_ALL, setlocale()

/*

 Help functions

*/

void
help()
{
    puts( gettext ("MP3T - The MP3 tag command line editor\n"
	  "(c) 2000 Martin Pitt <martin@piware.de>\n"
	  "This program is distributed under the terms and conditions of the\n"
	  "GNU General Public License.\n\n"
	  "Usage: mp3t [options] mp3file [mp3file ...]\n\n"
	  "  -r, --read[=FORMAT]\tRead out and print MP3 tag"
	  " (default). FORMAT: see below\n"
	  "  -w, --write\t\tWrite MP3 tag, data given at command line\n"
	  "\t\t\t(fields that are not specified are left untouched)\n"
	  "  -i, --input\t\tWrite MP3 tag, input data from console\n"
	  "  -d, --delete\t\tDelete MP3 tag\n\n"
	  "  --genres\t\tPrint all genres and their corresponding numbers and quit\n"
	  "  -h, --help\t\tPrint this help message and quit.\n"
	  "  --version\t\tPrint version information and quit.\n\n"
	  "Tag fields for write (-w / --write) mode:\n"
	  "  -t, --title=TITLE\tTitle field for write mode (max. 30 chars)\n"
	  "  -a, --album=ALBUM\tAlbum field for write mode (max. 30 chars)\n"
	  "  -n, --artist=ARTIST\tArtist (name) field for write mode (max. 30 chars)\n"
	  "  -y, --year=YEAR\tYear field for write mode (max. 4 chars)\n" "  -c, --comment=COMMENT\tComment field for write mode (max. 30 chars)\n"
	  "  -g, --genre=GENRE\tGenre field for write mode (number or string)\n\n"
	  "FORMAT: string to be printed out in read mode. You can use"
	  " the following variables:\n%TITLE, %ALBUM, %ARTIST, "
	  "%COMMENT, %YEAR, %GENRE, %FILE.\nEscapes are"
	  " recognized ( \\n \\r \\t \\\" \\\\ )\n\n"
	  "Example: mp3t -f \"Album %ALBUM was made"
	  " by:\\n%ARTIST in %YEAR\"\n\n"
	  "Please report bugs and suggestions to piware@piware.de\n") );
}

void
showGenres()
{
    for( int i = 0; i <= Tag::genre_largest; ++i )
	printf( "%3i %s\n", i, Tag::genres[i] );
}

/*

  Mode operation functions

*/

#define IFTAG(tag,fn,len) \
  if( !strncmp( format, tag, len ) ) { \
    fputs( t.get##fn(), stdout ); \
    format += len-1; }

void
printTag( const char* fname, char* format )
{
    Tag t;

    if( !readTag( fname, t ) )
	return;

    for(; *format; ++format ) {
	switch( *format ) {
	    case '%':
		if( !strncmp( ++format, "FILE", 4 ) ) {
		    fputs( fname, stdout );
		    format += 3; 
		}
		 else IFTAG( "TITLE", Title, 5 )
		 else IFTAG( "ALBUM", Album, 5 )
		 else IFTAG( "ARTIST", Artist, 6 )
		 else IFTAG( "COMMENT", Comment, 7 )
		 else IFTAG( "YEAR", Year, 4 )
		 else IFTAG( "GENRE", Genre, 5 )
		 else putchar( '%' );
		break;
		
	    case '\\':
		switch( *(++format) ) {
		    case 'n' : printf( "\n" ); break;
		    case 'r' : putchar( '\r' ); break;
		    case 't' : putchar( '\t' ); break;
		    case '"' : putchar( '"' ); break;
		    default  : putchar( *format ); break;
			 
		}
		break;
		
	    default:
		putchar( *format );
	}
    }
}

void
inputTag( const char* fname )
{
    char s[32];
    Tag t;
    int g;

    printf( gettext ("Enter data for file %s:\nTitle   : "), fname );
    inputString( s, 30 );
    t.setTitle( s );
    
    printf( gettext ("Album   : ") );
    inputString( s, 30 );
    t.setAlbum( s );
    
    printf( gettext ("Artist  : ") );
    inputString( s, 30 );
    t.setArtist( s );
    
    printf( gettext ("Year    : ") );
    inputString( s, 4 );
    t.setYear( s );

    printf( gettext ("Comment : ") );
    inputString( s, 30 );
    t.setComment( s );

    do {
	printf( gettext ("Genre    : ") );
	inputString( s, 23 );
	if( *s ) {
	    g = getGenre( s );
	    if( g == -1 )
		puts( gettext ("Unknown genre! Try again or just hit return for 'Unknown'") );
	}
	 else
	     g = Tag::genre_largest; // unknown
    } while( g == -1 );
	
    t.setGenre( g );

    puts( "" );

    writeTag( fname, t );
}


/*

  Main

*/

int
main( int argc, char** argv )
{
    Tag  t;
    int i;
	
    setlocale( LC_ALL, "" );
    bindtextdomain( PACKAGE, LOCALEDIR );
    textdomain( PACKAGE );

    if( argc == 1 ) {
	help();
	return 0;
    }

    if( !processOptions( argc, argv ) )
	return -1;

    switch( mode ) {
	case 'h': help (); return 0;
	case 'x': showGenres(); return 0;		 
	case 'v': puts( "mp3t 0.0.2 (02-18-2003)" ); return 0;

	case 'r' :
	    for( i = optind; i < argc; ++i )
		printTag( argv[i], format );
	    break;
	    
	case 'i':
	    for( i = optind; i < argc; ++i )
		inputTag( argv[i] );
	    break;

	case 'w':
	    for( i = optind; i < argc; ++i ) {
		readTag( argv[i], t );

		if( optt ) t.setTitle( opttag.getTitle() );
		if( opta ) t.setAlbum( opttag.getAlbum() );
		if( optn ) t.setArtist( opttag.getArtist() );
		if( opty ) t.setYear( opttag.getYear() );
		if( optc ) t.setComment( opttag.getComment() );
		if( optg ) t.setGenre( opttag.getGenreNum() );
		
		writeTag( argv[i], t );
	    }
	    break;

	case 'd':
	    for( i = optind; i < argc; ++i )
		deleteTag( argv[i] );
	    break;
    }
    
    return 0;
}
